package com.example.cs_openmediation_ad

import android.app.Activity
import androidx.annotation.NonNull
import com.openmediation.sdk.InitCallback
import com.openmediation.sdk.InitConfiguration
import com.openmediation.sdk.OmAds
import com.openmediation.sdk.interstitial.InterstitialAd
import com.openmediation.sdk.interstitial.InterstitialAdListener
import com.openmediation.sdk.splash.SplashAd
import com.openmediation.sdk.splash.SplashAdListener
import com.openmediation.sdk.utils.error.Error
import com.openmediation.sdk.utils.model.Scene
import com.openmediation.sdk.video.RewardedVideoAd
import com.openmediation.sdk.video.RewardedVideoListener
import com.openmediation.testsuite.TestSuite
import io.flutter.Log
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result

/** CsOpenmediationAdPlugin */
class CsOpenmediationAdPlugin: FlutterPlugin, MethodCallHandler, ActivityAware {
  /// The MethodChannel that will the communication between Flutter and native Android
  ///
  /// This local reference serves to register the plugin with the Flutter Engine and unregister it
  /// when the Flutter Engine is detached from the Activity
  private lateinit var channel: MethodChannel
  var activity: Activity? = null

  override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
    channel = MethodChannel(flutterPluginBinding.binaryMessenger, "cs_openmediation_ad")
    channel.setMethodCallHandler(this)
  }

  fun OnMethodCallHandler(method: String, args: Map<String, Any>?) {
    try {
      activity?.runOnUiThread(Runnable { channel.invokeMethod(method, args) });
    } catch (e: Exception) {
      Log.e("flutter_ayetstudios", "Error $e");
    }
  }

  fun getSceneMap(var1: Scene?) : Map<String, Any>? {
    var args: Map<String, Any>? = null;
    if (var1 != null) {
      args = mapOf<String, Any>("id" to var1.id, "n" to var1.n, "isd" to var1.isd, "oriData" to var1.oriData);
    }
    return args;
  }

  fun getSceneErrorMap(var1: Scene?, var2: Error?) : Map<String, Any>? {
    var id = 0;
    var n: String = "null";
    var isd = 0;
    var oriData: String = "null";
    var errorCode = -1;
    var errorMess = "";

    if (var1 != null) {
      id = var1.id;
      n = var1.n;
      isd = var1.isd;
      oriData = var1.oriData;
    }

    if (var2 != null) {
      var errorCode = -1;
      var errorMess = "";
    }
    return mapOf<String, Any>("id" to id, "n" to n, "isd" to isd, "oriData" to oriData, "errorCode" to errorCode, "errorMess" to errorMess);
  }

  fun setupListener() {
    RewardedVideoAd.setAdListener(object:RewardedVideoListener {
      override fun onRewardedVideoAvailabilityChanged(var1: Boolean) {
        val args: Map<String, Any> = mapOf<String, Any>("available" to var1);
        OnMethodCallHandler("rewardedVideoAvailabilityChanged", args);
      }
      override fun onRewardedVideoAdShowed(var1: Scene?) {
        OnMethodCallHandler("rewardedVideoAdShowed", getSceneMap(var1));
      }
      override fun onRewardedVideoAdShowFailed(var1: Scene?, var2: Error?) {
        OnMethodCallHandler("rewardedVideoAdShowFailed", getSceneErrorMap(var1, var2));
      }
      override fun onRewardedVideoAdClicked(var1: Scene?) {
        OnMethodCallHandler("rewardedVideoAdClicked", getSceneMap(var1));
      }
      override fun onRewardedVideoAdClosed(var1: Scene?) {
        OnMethodCallHandler("rewardedVideoAdClosed", getSceneMap(var1));
      }
      override fun onRewardedVideoAdStarted(var1: Scene?) {
        OnMethodCallHandler("rewardedVideoAdStarted", getSceneMap(var1));
      }
      override fun onRewardedVideoAdEnded(var1: Scene?) {
        OnMethodCallHandler("rewardedVideoAdEnded", getSceneMap(var1));
      }
      override fun onRewardedVideoAdRewarded(var1: Scene?) {
        OnMethodCallHandler("rewardedVideoAdRewarded", getSceneMap(var1));
      }
    });

    InterstitialAd.setAdListener(object:InterstitialAdListener{
      override fun onInterstitialAdAvailabilityChanged(var1: Boolean) {
        OnMethodCallHandler("interstitialAdAvailabilityChanged",  mapOf<String, Any>("availability" to var1));
      }
      override fun onInterstitialAdShowed(var1: Scene?){
        OnMethodCallHandler("interstitialAdShowed", getSceneMap(var1));
      }
      override fun onInterstitialAdShowFailed(var1: Scene?, var2: Error?) {
        OnMethodCallHandler("interstitialAdShowFailed", getSceneErrorMap(var1, var2));
      }
      override fun onInterstitialAdClosed(var1: Scene?) {
        OnMethodCallHandler("interstitialAdClosed", getSceneMap(var1));
      }
      override fun onInterstitialAdClicked(var1: Scene?) {
        OnMethodCallHandler("interstitialAdClicked", getSceneMap(var1));
      }
    });
  }

  fun setupSplashAdListener( placementId : String) {
    SplashAd.setSplashAdListener(placementId, object : SplashAdListener {
      override fun onSplashAdLoaded(placementId: String) {
        OnMethodCallHandler("splashAdLoaded", mapOf<String, Any>("placementId" to placementId));
      }
      override fun onSplashAdFailed(placementId: String, error: Error) {
        OnMethodCallHandler("splashAdFailed", mapOf<String, Any>("placementId" to placementId, "errorCode" to error.errorCode, "errorMess" to error.errorMessage));
      }
      override fun onSplashAdClicked(placementId: String) {
        OnMethodCallHandler("splashAdClicked", mapOf<String, Any>("placementId" to placementId));
      }
      override fun onSplashAdShowed(placementId: String) {
        OnMethodCallHandler("splashAdShowed", mapOf<String, Any>("placementId" to placementId));
      }
      override fun onSplashAdShowFailed(placementId: String, error: Error) {
        OnMethodCallHandler("splashAdShowFailed", mapOf<String, Any>("placementId" to placementId, "errorCode" to error.errorCode, "errorMess" to error.errorMessage));
      }
      override fun onSplashAdTick(placementId: String, millisUntilFinished: Long) {
        OnMethodCallHandler("splashAdTick", mapOf<String, Any>("placementId" to placementId, "millisUntilFinished" to millisUntilFinished));
      }
      override fun onSplashAdDismissed(placementId: String) {
        OnMethodCallHandler("splashAdDismissed", mapOf<String, Any>("placementId" to placementId));
      }
    });
  }

  override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
    if (call.method == "init") {
      val appKey: String = call.argument<String>("appKey") ?: return;
      val logEnable: Boolean = call.argument<Boolean>("logEnable") ?: return;
      setupListener();
      val configuration: InitConfiguration = InitConfiguration.Builder()
        .appKey(appKey)
        .preloadAdTypes(OmAds.AD_TYPE.REWARDED_VIDEO)
        .logEnable(logEnable)
        .build();
      println("kt appKey=$appKey, logEnable=$logEnable");
      OmAds.init(configuration, object : InitCallback {
        override fun onSuccess() {
          OnMethodCallHandler("initializationSuccess", null);
        }

        override fun onError(error: Error?) {
          var args: Map<String, Any>? = null;
          if (error != null) {
            args = mapOf<String, Any>("code" to error.errorCode, "message" to error.errorMessage);
          }
          OnMethodCallHandler("initializationFailed", args);
        }
      })
    } else if (call.method == "setUserId") {
      val userId: String = call.argument<String>("userId") ?: return;
      OmAds.setUserId(userId);
    } else if (call.method == "setExtId") {
      val sceneName: String = call.argument<String>("sceneName") ?: return;
      val extId: String = call.argument<String>("extId") ?: return;
      RewardedVideoAd.setExtId(sceneName, extId);
    } else if (call.method == "rewardVideoIsReady") {
      result.success(RewardedVideoAd.isReady());
    } else if (call.method == "showRewardVideo") {
      val sceneName: String? = call.argument<String>("sceneName");
      RewardedVideoAd.showAd(sceneName);
    } else if (call.method == "interstitialIsReady") {
      result.success(InterstitialAd.isReady());
    } else if (call.method == "interstitialIsSceneCapped") {
      val sceneName: String? = call.argument<String>("sceneName");
      result.success(InterstitialAd.isSceneCapped(sceneName));
    } else if (call.method == "showInterstitial") {
      val sceneName: String? = call.argument<String>("sceneName");
      InterstitialAd.showAd(sceneName);
    } else if (call.method == "loadSplashAd") {
      val placementId: String? = call.argument<String>("placementId");
      SplashAd.loadAd(placementId);
    } else if (call.method == "splashAdIsReady") {
      val placementId: String? = call.argument<String>("placementId");
      result.success(SplashAd.isReady(placementId));
    } else if (call.method == "showSplashAd") {
      val placementId: String? = call.argument<String>("placementId");
      SplashAd.showAd(placementId);
    } else if (call.method == "onResume") {
      OmAds.onResume(activity);
    } else if (call.method == "onPause") {
      OmAds.onPause(activity);
    } else if (call.method == "testSuite") {
      val appKey: String = call.argument<String>("appKey") ?: return;
      TestSuite.launch(activity, appKey);
    } else {
      result.notImplemented()
    }
  }

  override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
    channel.setMethodCallHandler(null);
  }

  override fun onAttachedToActivity(binding: ActivityPluginBinding) {
    activity = binding.activity;
  }

  override fun onDetachedFromActivityForConfigChanges() {}

  override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {
    activity = binding.activity;
  }

  override fun onDetachedFromActivity() {}
}
