import 'dart:developer';

import 'package:cs_openmediation_ad/cs_openmediation_model.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:cs_openmediation_ad/cs_openmediation_ad.dart';

void main() {
  runApp(const MyApp());
}

class MyApp extends StatefulWidget {
  const MyApp({Key? key}) : super(key: key);

  @override
  State<MyApp> createState() => _MyAppState();
}

class _MyAppState extends State<MyApp> {
  @override
  void initState() {
    super.initState();
    initSDK();
  }

  void initSDK() {
    CsOpenmediationAd().setInitializationSuccessListener(_initializationSuccess);
    CsOpenmediationAd().setInitilizationFailedListener(_initializationFailed);

    CsOpenmediationAd().setRewardedVideoAvailabilityChangedListener(_rewardedVideoAvailabilityChanged);
    CsOpenmediationAd().setRewardedVideoAdShowedListener(_rewardedVideoAdShowed);
    CsOpenmediationAd().setRewardedVideoAdShowFailedListener(_rewardedVideoAdShowFailed);
    CsOpenmediationAd().setRewardedVideoAdShowedListener(_rewardedVideoAdClicked);
    CsOpenmediationAd().setRewardedVideoAdShowedListener(_rewardedVideoAdClosed);
    CsOpenmediationAd().setRewardedVideoAdShowedListener(_rewardedVideoAdStarted);
    CsOpenmediationAd().setRewardedVideoAdShowedListener(_rewardedVideoAdEnded);
    CsOpenmediationAd().setRewardedVideoAdShowedListener(_rewardedVideoAdRewarded);

    // 插屏广告
    CsOpenmediationAd().setInterstitialAdAvailabilityChanged(_interstitialAdAvailabilityChanged);
    CsOpenmediationAd().setInterstitialAdShowed(_interstitialAdShowed);
    CsOpenmediationAd().setInterstitialAdShowFailed(_interstitialAdShowFailed);
    CsOpenmediationAd().setInterstitialAdClosed(_interstitialAdClosed);
    CsOpenmediationAd().setInterstitialAdClicked(_interstitialAdClicked);

    //开屏广告
    CsOpenmediationAd().setSplashAdLoaded(_splashAdLoaded);
    CsOpenmediationAd().setSplashAdFailed(_splashAdFailed);
    CsOpenmediationAd().setSplashAdClicked(_splashAdClicked);
    CsOpenmediationAd().setSplashAdShowed(_splashAdShowed);
    CsOpenmediationAd().setSplashAdShowFailed(_splashAdShowFailed);
    CsOpenmediationAd().setSplashAdTick(_splashAdTick);
    CsOpenmediationAd().setSplashAdDismissed(_splashAdDismissed);

    // CsOpenmediationAd().init("gwsQFNL6ef0WZoIWP1rtHg0gokviPjOX", !kReleaseMode); //测试版
  }

  _initializationSuccess() {
    log("OpenMediation Callback: _initializationSuccess");
  }

  _initializationFailed(CSOMError? error) {
    log("OpenMediation Callback: _initializationFailed");
  }

  _rewardedVideoAvailabilityChanged(bool available) {
    log("OpenMediation Callback: _rewardedVideoAvailabilityChanged");
  }

  _rewardedVideoAdShowed(CSOMScene? scene) {
    log("OpenMediation Callback: _rewardedVideoAdShowed");
  }

  _rewardedVideoAdShowFailed(CSOMScene? scene, CSOMError? error) {
    log("OpenMediation Callback: _rewardedVideoAdShowFailed");
  }

  _rewardedVideoAdClicked(CSOMScene? scene) {
    log("OpenMediation Callback: _rewardedVideoAdClicked");
  }

  _rewardedVideoAdClosed(CSOMScene? scene) {
    log("OpenMediation Callback: _rewardedVideoAdClosed");
  }

  _rewardedVideoAdStarted(CSOMScene? scene) {
    log("OpenMediation Callback: _rewardedVideoAdStarted");
  }

  _rewardedVideoAdEnded(CSOMScene? scene) {
    log("OpenMediation Callback: _rewardedVideoAdEnded");
  }

  _rewardedVideoAdRewarded(CSOMScene? scene) {
    log("OpenMediation Callback: _rewardedVideoAdRewarded");
  }

  // 插屏广告
  _interstitialAdAvailabilityChanged(bool available) {
    log("OpenMediation Callback: _interstitialAdAvailabilityChanged");
  }

  _interstitialAdShowed(CSOMScene? scene) {
    log("OpenMediation Callback: _interstitialAdShowed");
  }

  _interstitialAdShowFailed(CSOMScene? scene, CSOMError? error) {
    log("OpenMediation Callback: _interstitialAdShowFailed");
  }

  _interstitialAdClosed(CSOMScene? scene) {
    log("OpenMediation Callback: _interstitialAdClosed");
  }

  _interstitialAdClicked(CSOMScene? scene) {
    log("OpenMediation Callback: _interstitialAdClicked");
  }

  //开屏广告
  _splashAdLoaded(String placementId) {
    log("OpenMediation Callback: _splashAdLoaded");
  }

  _splashAdFailed(String placementId, CSOMError? error) {
    log("OpenMediation Callback: _splashAdFailed");
  }

  _splashAdClicked(String placementId) {
    log("OpenMediation Callback: _splashAdClicked");
  }

  _splashAdShowed(String placementId) {
    log("OpenMediation Callback: _splashAdShowed");
  }

  _splashAdShowFailed(String placementId, CSOMError? error) {
    log("OpenMediation Callback: _splashAdShowFailed");
  }

  _splashAdTick(String placementId, int millisUntilFinished) {
    log("OpenMediation Callback: _splashAdTick");
  }

  _splashAdDismissed(String placementId) {
    log("OpenMediation Callback: _splashAdDismissed");
  }

  void showAd() {
    CsOpenmediationAd().showTest("gwsQFNL6ef0WZoIWP1rtHg0gokviPjOX");
  }

  @override
  Widget build(BuildContext context) {
    return MaterialApp(
      home: Scaffold(
        appBar: AppBar(
          title: const Text('Plugin example app'),
        ),
        body: Center(
          child: TextButton(onPressed: () => showAd(), child: const Text('show ad', style: TextStyle(color: Colors.red))),
        ),
      ),
    );
  }
}
