import 'dart:async';
import 'dart:developer';

import 'package:flutter/services.dart';

import 'cs_openmediation_model.dart';

typedef InitializationSuccess = void Function();
typedef InitializationFailed = void Function(CSOMError? error);

// 激励视频
typedef RewardedVideoAvailabilityChanged = void Function(bool available);
typedef RewardedVideoAdShowed = void Function(CSOMScene? scene);
typedef RewardedVideoAdShowFailed = void Function(CSOMScene? scene, CSOMError? error);
typedef RewardedVideoAdClicked = void Function(CSOMScene? scene);
typedef RewardedVideoAdClosed = void Function(CSOMScene? scene);
typedef RewardedVideoAdStarted = void Function(CSOMScene? scene);
typedef RewardedVideoAdEnded = void Function(CSOMScene? scene);
typedef RewardedVideoAdRewarded = void Function(CSOMScene? scene);

// 插屏广告
typedef InterstitialAdAvailabilityChanged = void Function(bool available);
typedef InterstitialAdShowed = void Function(CSOMScene? scene);
typedef InterstitialAdShowFailed = void Function(CSOMScene? scene, CSOMError? error);
typedef InterstitialAdClosed = void Function(CSOMScene? scene);
typedef InterstitialAdClicked = void Function(CSOMScene? scene);

//开屏广告
typedef SplashAdLoaded = void Function(String placementId);
typedef SplashAdFailed = void Function(String placementId, CSOMError? error);
typedef SplashAdClicked = void Function(String placementId);
typedef SplashAdShowed = void Function(String placementId);
typedef SplashAdShowFailed = void Function(String placementId, CSOMError? error);
typedef SplashAdTick = void Function(String placementId, int millisUntilFinished);
typedef SplashAdDismissed = void Function(String placementId);

class CsOpenmediationAd {
  static const MethodChannel _channel = MethodChannel('cs_openmediation_ad');
  static final CsOpenmediationAd _instance = CsOpenmediationAd._internal();
  factory CsOpenmediationAd() => _instance;
  CsOpenmediationAd._internal() {
    _channel.setMethodCallHandler(_platformCallHandler);
  }

  late InitializationSuccess _initializationSuccess;
  late InitializationFailed _initializationFailed;

  // 激励视频
  late RewardedVideoAvailabilityChanged _rewardedVideoAvailabilityChanged;
  late RewardedVideoAdShowed _rewardedVideoAdShowed;
  late RewardedVideoAdShowFailed _rewardedVideoAdShowFailed;
  late RewardedVideoAdClicked _rewardedVideoAdClicked;
  late RewardedVideoAdClosed _rewardedVideoAdClosed;
  late RewardedVideoAdStarted _rewardedVideoAdStarted;
  late RewardedVideoAdEnded _rewardedVideoAdEnded;
  late RewardedVideoAdRewarded _rewardedVideoAdRewarded;

  // 插屏广告
  late InterstitialAdAvailabilityChanged _interstitialAdAvailabilityChanged;
  late InterstitialAdShowed _interstitialAdShowed;
  late InterstitialAdShowFailed _interstitialAdShowFailed;
  late InterstitialAdClosed _interstitialAdClosed;
  late InterstitialAdClicked _interstitialAdClicked;

  //开屏广告
  late SplashAdLoaded _splashAdLoaded;
  late SplashAdFailed _splashAdFailed;
  late SplashAdClicked _splashAdClicked;
  late SplashAdShowed _splashAdShowed;
  late SplashAdShowFailed _splashAdShowFailed;
  late SplashAdTick _splashAdTick;
  late SplashAdDismissed _splashAdDismissed;

  CSOMScene? _getSceneFromArgumeennts(dynamic arguments) {
    if (arguments["id"] != null) {
      int id = arguments["id"];
      String? n = arguments["n"];
      int? isd = arguments["isd"];
      String? oriData = arguments["oriData"];
      return CSOMScene(id, n, isd, oriData);
    }
    return null;
  }

  CSOMError? _getErrorFromArgumeennts(dynamic arguments) {
    if (arguments["errorCode"] != null) {
      int code = arguments["errorCode"];
      String? message = arguments["errorMess"];
      return CSOMError(code, message);
    }
    return null;
  }

  Future _platformCallHandler(MethodCall call) async {
    switch (call.method) {
      case "initializationSuccess":
        _initializationSuccess();
        break;
      case "initializationFailed":
        CSOMError? error = _getErrorFromArgumeennts(call.arguments);
        _initializationFailed(error);
        break;
      case "rewardedVideoAvailabilityChanged":
        bool available = call.arguments["available"];
        _rewardedVideoAvailabilityChanged(available);
        break;
      case "rewardedVideoAdShowed":
        var scene = _getSceneFromArgumeennts(call.arguments);
        _rewardedVideoAdShowed(scene);
        break;
      case "rewardedVideoAdShowFailed":
        CSOMScene? scene = _getSceneFromArgumeennts(call.arguments);
        CSOMError? error = _getErrorFromArgumeennts(call.arguments);
        _rewardedVideoAdShowFailed(scene, error);
        break;
      case "rewardedVideoAdClicked":
        CSOMScene? scene = _getSceneFromArgumeennts(call.arguments);
        _rewardedVideoAdClicked(scene);
        break;
      case "rewardedVideoAdClosed":
        CSOMScene? scene = _getSceneFromArgumeennts(call.arguments);
        _rewardedVideoAdClosed(scene);
        break;
      case "rewardedVideoAdStarted":
        CSOMScene? scene = _getSceneFromArgumeennts(call.arguments);
        _rewardedVideoAdStarted(scene);
        break;
      case "rewardedVideoAdEnded":
        CSOMScene? scene = _getSceneFromArgumeennts(call.arguments);
        _rewardedVideoAdEnded(scene);
        break;
      case "rewardedVideoAdRewarded":
        CSOMScene? scene = _getSceneFromArgumeennts(call.arguments);
        _rewardedVideoAdRewarded(scene);
        break;

      case "interstitialAdAvailabilityChanged":
        bool available = call.arguments["available"];
        _interstitialAdAvailabilityChanged(available);
        break;
      case "interstitialAdShowed":
        CSOMScene? scene = _getSceneFromArgumeennts(call.arguments);
        _interstitialAdShowed(scene);
        break;
      case "interstitialAdShowFailed":
        CSOMScene? scene = _getSceneFromArgumeennts(call.arguments);
        CSOMError? error = _getErrorFromArgumeennts(call.arguments);
        _interstitialAdShowFailed(scene, error);
        break;
      case "interstitialAdClosed":
        CSOMScene? scene = _getSceneFromArgumeennts(call.arguments);
        _interstitialAdClosed(scene);
        break;
      case "interstitialAdClicked":
        CSOMScene? scene = _getSceneFromArgumeennts(call.arguments);
        _interstitialAdClicked(scene);
        break;

      case "splashAdLoaded":
        String placementId = call.arguments["placementId"];
        _splashAdLoaded(placementId);
        break;
      case "splashAdFailed":
        String placementId = call.arguments["placementId"];
        CSOMError? error = _getErrorFromArgumeennts(call.arguments);
        _splashAdFailed(placementId, error);
        break;
      case "splashAdClicked":
        String placementId = call.arguments["placementId"];
        _splashAdClicked(placementId);
        break;
      case "splashAdShowed":
        String placementId = call.arguments["placementId"];
        _splashAdShowed(placementId);
        break;
      case "splashAdShowFailed":
        String placementId = call.arguments["placementId"];
        CSOMError? error = _getErrorFromArgumeennts(call.arguments);
        _splashAdShowFailed(placementId, error);
        break;
      case "splashAdTick":
        String placementId = call.arguments["placementId"];
        int millisUntilFinished = call.arguments["millisUntilFinished"];
        _splashAdTick(placementId, millisUntilFinished);
        break;
      case "splashAdDismissed":
        String placementId = call.arguments["placementId"];
        _splashAdDismissed(placementId);
        break;

      default:
        log('Unknown method ${call.method} ');
    }
  }

  /// 初始话SDK
  Future<void> init(String appKey, bool logEnable) {
    return _channel.invokeMethod("init", <String, dynamic>{
      'appKey': appKey,
      'logEnable': logEnable
    });
  }

  /// 设置用户名
  Future<void> setUserId(String userId) {
    return _channel.invokeMethod("setUserId", <String, dynamic>{
      'userId': userId,
    });
  }

  /// 设置ExtId
  Future<void> setExtId(String sceneName, String extId) {
    return _channel.invokeMethod("setExtId", <String, dynamic>{
      'sceneName': sceneName,
      'extId': extId,
    });
  }

  /// 展示激励视频
  Future<void> showRewardVideo(String sceneName) {
    return _channel.invokeMethod("showRewardVideo", <String, dynamic>{
      'sceneName': sceneName,
    });
  }

  /// 激励视频是否准备好了
  Future<bool?> rewardVideoIsReady() {
    return _channel.invokeMethod<bool>("rewardVideoIsReady");
  }

  /// 展示插页广告
  Future<void> showInterstitial(String sceneName) {
    return _channel.invokeMethod("showInterstitial", <String, dynamic>{
      'sceneName': sceneName,
    });
  }

  /// 展示插页广告
  Future<void> interstitialIsSceneCapped(String sceneName) {
    return _channel.invokeMethod("interstitialIsSceneCapped", <String, dynamic>{
      'sceneName': sceneName,
    });
  }

  /// 插页广告是否准备好了
  Future<bool?> interstitialIsReady() {
    return _channel.invokeMethod<bool>("interstitialIsReady");
  }

  /// 获取开屏广告
  Future<void> loadSplashAd(String placementId) {
    return _channel.invokeMethod("loadSplashAd", <String, dynamic>{
      'placementId': placementId,
    });
  }

  /// 展示开屏广告
  Future<void> showSplashAd(String placementId) {
    return _channel.invokeMethod("showSplashAd", <String, dynamic>{
      'placementId': placementId,
    });
  }

  /// 开屏广告是否准备好了
  Future<bool?> splashAdIsReady(String placementId) {
    return _channel.invokeMethod<bool>("splashAdIsReady", <String, dynamic>{
      'placementId': placementId,
    });
  }

  /// 展示测试版本
  Future<void> showTest(String appKey) {
    return _channel.invokeMethod<void>("testSuite", <String, dynamic>{
      'appKey': appKey,
    });
  }

  Future<void> onResume() {
    return _channel.invokeMethod("onResume");
  }

  Future<void> onPause() {
    return _channel.invokeMethod("onPause");
  }

  /// SDK初始化成功
  void setInitializationSuccessListener(InitializationSuccess initializationSuccess) => _initializationSuccess = initializationSuccess;

  /// SDK初始化失败
  void setInitilizationFailedListener(InitializationFailed initializationFailed) => _initializationFailed = initializationFailed;

  //激励视频
  void setRewardedVideoAvailabilityChangedListener(RewardedVideoAvailabilityChanged rewardedVideoAvailabilityChanged) => _rewardedVideoAvailabilityChanged = rewardedVideoAvailabilityChanged;
  void setRewardedVideoAdShowedListener(RewardedVideoAdShowed rewardedVideoAdShowed) => _rewardedVideoAdShowed = rewardedVideoAdShowed;
  void setRewardedVideoAdShowFailedListener(RewardedVideoAdShowFailed rewardedVideoAdShowFailed) => _rewardedVideoAdShowFailed = rewardedVideoAdShowFailed;
  void setRewardedVideoAdClickedListener(RewardedVideoAdClicked rewardedVideoAdClicked) => _rewardedVideoAdClicked = rewardedVideoAdClicked;
  void setRewardedVideoAdClosedListener(RewardedVideoAdClosed rewardedVideoAdClosed) => _rewardedVideoAdClosed = rewardedVideoAdClosed;
  void setRewardedVideoAdStartedListener(RewardedVideoAdStarted rewardedVideoAdStarted) => _rewardedVideoAdStarted = rewardedVideoAdStarted;
  void setRewardedVideoAdEndedListener(RewardedVideoAdEnded rewardedVideoAdEnded) => _rewardedVideoAdEnded = rewardedVideoAdEnded;
  void setRewardedVideoAdRewardedListener(RewardedVideoAdRewarded rewardedVideoAdRewarded) => _rewardedVideoAdRewarded = rewardedVideoAdRewarded;

  // 插屏广告
  void setInterstitialAdAvailabilityChanged(InterstitialAdAvailabilityChanged interstitialAdAvailabilityChanged) => _interstitialAdAvailabilityChanged = interstitialAdAvailabilityChanged;
  void setInterstitialAdShowed(InterstitialAdShowed interstitialAdShowed) => _interstitialAdShowed = interstitialAdShowed;
  void setInterstitialAdShowFailed(InterstitialAdShowFailed interstitialAdShowFailed) => _interstitialAdShowFailed = interstitialAdShowFailed;
  void setInterstitialAdClosed(InterstitialAdClosed interstitialAdClosed) => _interstitialAdClosed = interstitialAdClosed;
  void setInterstitialAdClicked(InterstitialAdClicked interstitialAdClicked) => _interstitialAdClicked = interstitialAdClicked;

  //开屏广告
  void setSplashAdLoaded(SplashAdLoaded splashAdLoaded) => _splashAdLoaded = splashAdLoaded;
  void setSplashAdFailed(SplashAdFailed splashAdFailed) => _splashAdFailed = splashAdFailed;
  void setSplashAdClicked(SplashAdClicked splashAdClicked) => _splashAdClicked = splashAdClicked;
  void setSplashAdShowed(SplashAdShowed splashAdShowed) => _splashAdShowed = splashAdShowed;
  void setSplashAdShowFailed(SplashAdShowFailed splashAdShowFailed) => _splashAdShowFailed = splashAdShowFailed;
  void setSplashAdTick(SplashAdTick splashAdTick) => _splashAdTick = splashAdTick;
  void setSplashAdDismissed(SplashAdDismissed splashAdDismissed) => _splashAdDismissed = splashAdDismissed;
}
